@@ -1,7 +1,7 @@
 /* ebrowse.c --- parsing files for the ebrowse C++ browser
 
 Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-              2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
+              2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
               Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
@@ -3680,7 +3680,7 @@ void
 version ()
 {
   /* Makes it easier to update automatically. */
-  char emacs_copyright[] = "Copyright (C) 2009 Free Software Foundation, Inc.";
+  char emacs_copyright[] = "Copyright (C) 2010 Free Software Foundation, Inc.";
 
   printf ("ebrowse %s\n", VERSION);
   puts (emacs_copyright);
