@@ -1,6 +1,6 @@
 /* sendmail-like interface to /bin/mail for system V,
    Copyright (C) 1985, 1994, 1999, 2001, 2002, 2003, 2004,
-                 2005, 2006, 2007, 2008, 2009  Free Software Foundation, Inc.
+                 2005, 2006, 2007, 2008, 2009, 2010  Free Software Foundation, Inc.
 
 Author: Bill Rozas <jinx@martigny.ai.mit.edu>
 (according to ack.texi)
