@@ -1,17 +1,17 @@
 ;;; calc-mode.el --- calculator modes for Calc
 
 ;; Copyright (C) 1990, 1991, 1992, 1993, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+;;   2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 
 ;; Author: David Gillespie <daveg@synaptics.com>
 ;; Maintainer: Jay Belanger <jay.p.belanger@gmail.com>
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 3, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +19,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -32,6 +30,10 @@
 (require 'calc-ext)
 (require 'calc-macs)
 
+;; Declare functions which are defined elsewhere.
+(declare-function calc-embedded-save-original-modes "calc-embed" ())
+
+
 (defun calc-line-numbering (n)
   (interactive "P")
   (calc-wrapper
@@ -501,7 +503,7 @@
 			     mode)
 		      (and (not (eq calc-simplify-mode mode))
 			   mode)))
-  (message (if (eq calc-simplify-mode mode)
+  (message "%s" (if (eq calc-simplify-mode mode)
 	       msg
 	     "Default simplifications enabled")))
 
@@ -662,7 +664,7 @@
      (message "Not surrounding vectors with brackets"))))
 
 (defun calc-matrix-brackets (arg)
-  (interactive "sCode letters (R, O, C, P): ")
+  (interactive "sCode letters (R, O, C): ")
   (calc-wrapper
    (let ((code (append (and (string-match "[rR]" arg) '(R))
 		       (and (string-match "[oO]" arg) '(O))
@@ -675,5 +677,5 @@
 
 (provide 'calc-mode)
 
-;;; arch-tag: ecc70eea-c712-43f2-9085-4205e58d6ddf
+;; arch-tag: ecc70eea-c712-43f2-9085-4205e58d6ddf
 ;;; calc-mode.el ends here
