@@ -1,17 +1,18 @@
 ;;; pcvs-defs.el --- variable definitions for PCL-CVS
 
 ;; Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-;;   2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+;;   2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+;;   Free Software Foundation, Inc.
 
 ;; Author: Stefan Monnier <monnier@iro.umontreal.ca>
 ;; Keywords: pcl-cvs
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 3, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +20,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -76,7 +75,7 @@ versions, such as the one in SunOS-4.")
 ;;  cvsrc options
 ;;
 
-(defcustom cvs-cvsrc-file "~/.cvsrc"
+(defcustom cvs-cvsrc-file (convert-standard-filename "~/.cvsrc")
   "Path to your cvsrc file."
   :group 'pcl-cvs
   :type '(file))
@@ -142,7 +141,8 @@ current line.  See also `cvs-invert-ignore-marks'"
 
 (defvar cvs-diff-ignore-marks t)
 (make-obsolete-variable 'cvs-diff-ignore-marks
-                        'cvs-invert-ignore-marks)
+                        'cvs-invert-ignore-marks
+			"21.1")
 
 (defcustom cvs-invert-ignore-marks
   (let ((l ()))
@@ -179,7 +179,8 @@ If set to nil, `cvs-mode-add' will always prompt for a message."
 
 (defvar cvs-diff-buffer-name "*cvs-diff*")
 (make-obsolete-variable 'cvs-diff-buffer-name
-                        'cvs-buffer-name-alist)
+                        'cvs-buffer-name-alist
+			"21.1")
 
 (defcustom cvs-find-file-and-jump nil
   "Jump to the modified area when finding a file.
@@ -320,11 +321,7 @@ This variable is buffer local and only used in the *cvs* buffer.")
 (easy-mmode-defmap cvs-mode-map
   ;;(define-prefix-command 'cvs-mode-map-diff-prefix)
   ;;(define-prefix-command 'cvs-mode-map-control-c-prefix)
-  '(;; simulate `suppress-keymap'
-    (self-insert-command . undefined)
-    (("0" "1" "2" "3" "4" "5" "6" "7" "8" "9") . digit-argument)
-    ("-" .	negative-argument)
-    ;; various
+  '(;; various
     ;; (undo .	cvs-mode-undo)
     ("?" .	cvs-help)
     ("h" .	cvs-help)
@@ -398,14 +395,15 @@ This variable is buffer local and only used in the *cvs* buffer.")
     ;;([tool-bar item2] . (menu-item "Update" cvs-update :image (image :file "/usr/share/icons/mail1.xpm" :type xpm)))
     )
   "Keymap for `cvs-mode'."
-  :dense t)
+  :dense t
+  :suppress t)
 
 (fset 'cvs-mode-map cvs-mode-map)
 
 (easy-menu-define cvs-menu cvs-mode-map "Menu used in `cvs-mode'."
   '("CVS"
-    ["Open file.."		cvs-mode-find-file	t]
-    [" ..other window"		cvs-mode-find-file-other-window	t]
+    ["Open file"		cvs-mode-find-file	t]
+    ["Open in other window"	cvs-mode-find-file-other-window	t]
     ["Display in other window"  cvs-mode-display-file   t]
     ["Interactive merge"	cvs-mode-imerge		t]
     ("View diff"
@@ -413,6 +411,7 @@ This variable is buffer local and only used in the *cvs* buffer.")
      ["Current diff"		cvs-mode-diff		t]
      ["Diff with head"		cvs-mode-diff-head	t]
      ["Diff with vendor"	cvs-mode-diff-vendor	t]
+     ["Diff against yesterday"	cvs-mode-diff-yesterday	t]
      ["Diff with backup"	cvs-mode-diff-backup	t])
     ["View log"			cvs-mode-log		t]
     ["View status"		cvs-mode-status		t]
@@ -437,6 +436,9 @@ This variable is buffer local and only used in the *cvs* buffer.")
     ["Unmark all"		cvs-mode-unmark-all-files t]
     ["Hide handled"		cvs-mode-remove-handled	t]
     "----"
+    ["PCL-CVS Manual"		(lambda () (interactive)
+				  (info "(pcl-cvs)Top")) t]
+    "----"
     ["Quit"			cvs-mode-quit		t]))
 
 ;;;;
@@ -481,17 +483,17 @@ It is expected to call the function.")
 (defvar cvs-global-menu
   (let ((m (make-sparse-keymap "PCL-CVS")))
     (define-key m [status]
-      '(menu-item "Directory Status" cvs-status
-		  :help "A more verbose status of a workarea"))
+      `(menu-item ,(purecopy "Directory Status") cvs-status
+		  :help ,(purecopy "A more verbose status of a workarea")))
     (define-key m [checkout]
-      '(menu-item "Checkout Module" cvs-checkout
-		  :help "Check out a module from the repository"))
+      `(menu-item ,(purecopy "Checkout Module") cvs-checkout
+		  :help ,(purecopy "Check out a module from the repository")))
     (define-key m [update]
-      '(menu-item "Update Directory" cvs-update
-		  :help "Fetch updates from the repository"))
+      `(menu-item ,(purecopy "Update Directory") cvs-update
+		  :help ,(purecopy "Fetch updates from the repository")))
     (define-key m [examine]
-      '(menu-item "Examine Directory" cvs-examine
-		  :help "Examine the current state of a workarea"))
+      `(menu-item ,(purecopy "Examine Directory") cvs-examine
+		  :help ,(purecopy "Examine the current state of a workarea")))
     (fset 'cvs-global-menu m)))
 
 
