@@ -1,17 +1,18 @@
 ;;; pcvs-info.el --- internal representation of a fileinfo entry
 
 ;; Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-;;   2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+;;   2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+;;   Free Software Foundation, Inc.
 
 ;; Author: Stefan Monnier <monnier@iro.umontreal.ca>
 ;; Keywords: pcl-cvs
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 3, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +20,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -41,16 +40,18 @@
 ;;;; config variables
 ;;;;
 
+(define-obsolete-variable-alias 'cvs-display-full-path
+    'cvs-display-full-name "22.1")
+
 (defcustom cvs-display-full-name t
-  "*Specifies how the filenames should be displayed in the listing.
+  "Specifies how the filenames should be displayed in the listing.
 If non-nil, their full filename name will be displayed, else only the
 non-directory part."
   :group 'pcl-cvs
   :type '(boolean))
-(define-obsolete-variable-alias 'cvs-display-full-path 'cvs-display-full-name)
 
 (defcustom cvs-allow-dir-commit nil
-  "*Allow `cvs-mode-commit' on directories.
+  "Allow `cvs-mode-commit' on directories.
 If you commit without any marked file and with the cursor positioned
 on a directory entry, cvs would commit the whole directory.  This seems
 to confuse some users sometimes."
@@ -69,8 +70,7 @@ to confuse some users sometimes."
     (t (:weight bold)))
   "PCL-CVS face used to highlight directory changes."
   :group 'pcl-cvs)
-;; backward-compatibility alias
-(put 'cvs-header-face 'face-alias 'cvs-header)
+(define-obsolete-face-alias 'cvs-header-face 'cvs-header "22.1")
 
 (defface cvs-filename
   '((((class color) (background dark))
@@ -80,8 +80,7 @@ to confuse some users sometimes."
     (t ()))
   "PCL-CVS face used to highlight file names."
   :group 'pcl-cvs)
-;; backward-compatibility alias
-(put 'cvs-filename-face 'face-alias 'cvs-filename)
+(define-obsolete-face-alias 'cvs-filename-face 'cvs-filename "22.1")
 
 (defface cvs-unknown
   '((((class color) (background dark))
@@ -91,8 +90,7 @@ to confuse some users sometimes."
     (t (:slant italic)))
   "PCL-CVS face used to highlight unknown file status."
   :group 'pcl-cvs)
-;; backward-compatibility alias
-(put 'cvs-unknown-face 'face-alias 'cvs-unknown)
+(define-obsolete-face-alias 'cvs-unknown-face 'cvs-unknown "22.1")
 
 (defface cvs-handled
   '((((class color) (background dark))
@@ -102,8 +100,7 @@ to confuse some users sometimes."
     (t ()))
   "PCL-CVS face used to highlight handled file status."
   :group 'pcl-cvs)
-;; backward-compatibility alias
-(put 'cvs-handled-face 'face-alias 'cvs-handled)
+(define-obsolete-face-alias 'cvs-handled-face 'cvs-handled "22.1")
 
 (defface cvs-need-action
   '((((class color) (background dark))
@@ -113,8 +110,7 @@ to confuse some users sometimes."
     (t (:slant italic)))
   "PCL-CVS face used to highlight status of files needing action."
   :group 'pcl-cvs)
-;; backward-compatibility alias
-(put 'cvs-need-action-face 'face-alias 'cvs-need-action)
+(define-obsolete-face-alias 'cvs-need-action-face 'cvs-need-action "22.1")
 
 (defface cvs-marked
   '((((min-colors 88) (class color) (background dark))
@@ -126,15 +122,13 @@ to confuse some users sometimes."
     (t (:weight bold)))
   "PCL-CVS face used to highlight marked file indicator."
   :group 'pcl-cvs)
-;; backward-compatibility alias
-(put 'cvs-marked-face 'face-alias 'cvs-marked)
+(define-obsolete-face-alias 'cvs-marked-face 'cvs-marked "22.1")
 
 (defface cvs-msg
   '((t (:slant italic)))
   "PCL-CVS face used to highlight CVS messages."
   :group 'pcl-cvs)
-;; backward-compatibility alias
-(put 'cvs-msg-face 'face-alias 'cvs-msg)
+(define-obsolete-face-alias 'cvs-msg-face 'cvs-msg "22.1")
 
 (defvar cvs-fi-up-to-date-face 'cvs-handled)
 (defvar cvs-fi-unknown-face 'cvs-unknown)
@@ -225,7 +219,8 @@ to confuse some users sometimes."
       ;; Here, I use `concat' rather than `expand-file-name' because I want
       ;; the resulting path to stay relative if `dir' is relative.
       (concat dir (cvs-fileinfo->file fileinfo)))))
-(define-obsolete-function-alias 'cvs-fileinfo->full-path 'cvs-fileinfo->full-name)
+(define-obsolete-function-alias 'cvs-fileinfo->full-path
+    'cvs-fileinfo->full-name "22.1")
 
 (defun cvs-fileinfo->pp-name (fi)
   "Return the filename of FI as it should be displayed."
